@@ -192,10 +192,11 @@ typedef enum {
 
 
 /**
- * These ISO, shutter speed, aperture, etc. settings are correct for the 
- * EOS 5D; unsure about other cameras.
+ * These ISO, shutter speed, aperture, etc. settings are correct for the
+ * EOS 5D and some for the G1; unsure about other cameras.
  */
 typedef enum {
+	ISO_50 = 0x40,
 	ISO_100 = 0x48,
 	ISO_125 = 0x4b,
 	ISO_160 = 0x4d,
@@ -217,7 +218,13 @@ struct canonIsoStateStruct {
 	char *label;
 };
 
+struct canonShootingModeStateStruct {
+	unsigned char value;
+	char *label;
+};
+
 typedef enum {
+	APERTURE_F1_2 = 0x0d,
 	APERTURE_F1_4 = 0x10,
 	APERTURE_F1_6 = 0x13,
 	APERTURE_F1_8 = 0x15,
@@ -318,24 +325,24 @@ struct canonShutterSpeedStateStruct {
 };
 
 typedef enum {
-	RESOLUTION_RAW                        = 0,
-	RESOLUTION_RAW_AND_LARGE_FINE_JPEG, 
-	RESOLUTION_RAW_AND_LARGE_NORMAL_JPEG, 
-	RESOLUTION_RAW_AND_MEDIUM_FINE_JPEG, 
-	RESOLUTION_RAW_AND_MEDIUM_NORMAL_JPEG,
-	RESOLUTION_RAW_AND_SMALL_FINE_JPEG,
-	RESOLUTION_RAW_AND_SMALL_NORMAL_JPEG,
-	RESOLUTION_LARGE_FINE_JPEG,
-	RESOLUTION_LARGE_NORMAL_JPEG,
-	RESOLUTION_MEDIUM_FINE_JPEG,
-	RESOLUTION_MEDIUM_NORMAL_JPEG,
-	RESOLUTION_SMALL_FINE_JPEG,
-	RESOLUTION_SMALL_NORMAL_JPEG
-} canonResolutionState;
+	IMAGE_FORMAT_RAW                        = 0,
+	IMAGE_FORMAT_RAW_AND_LARGE_FINE_JPEG,
+	IMAGE_FORMAT_RAW_AND_LARGE_NORMAL_JPEG,
+	IMAGE_FORMAT_RAW_AND_MEDIUM_FINE_JPEG,
+	IMAGE_FORMAT_RAW_AND_MEDIUM_NORMAL_JPEG,
+	IMAGE_FORMAT_RAW_AND_SMALL_FINE_JPEG,
+	IMAGE_FORMAT_RAW_AND_SMALL_NORMAL_JPEG,
+	IMAGE_FORMAT_LARGE_FINE_JPEG,
+	IMAGE_FORMAT_LARGE_NORMAL_JPEG,
+	IMAGE_FORMAT_MEDIUM_FINE_JPEG,
+	IMAGE_FORMAT_MEDIUM_NORMAL_JPEG,
+	IMAGE_FORMAT_SMALL_FINE_JPEG,
+	IMAGE_FORMAT_SMALL_NORMAL_JPEG
+} canonImageFormatState;
 
 
-struct canonResolutionStateStruct {
-	canonResolutionState value;
+struct canonImageFormatStateStruct {
+	canonImageFormatState value;
 	char *label;
 	unsigned char res_byte1;
 	unsigned char res_byte2;
@@ -386,25 +393,8 @@ struct canonBeepModeStateStruct {
 	char* label;
 };
 
-/*
- * Controls che camera zoom. These values have been verified for the
- * A40 only.
- */
-typedef enum {
-	ZOOM_0 = 0x01, /* minimum (no zoom) */
-	ZOOM_1 = 0x02,
-	ZOOM_2 = 0x03,
-	ZOOM_3 = 0x04,
-	ZOOM_4 = 0x05,
-	ZOOM_5 = 0x06,
-	ZOOM_6 = 0x07,
-	ZOOM_7 = 0x08,
-	ZOOM_8 = 0x09,	
-	ZOOM_9 = 0x0a /* maximum zoom ?? */
-} canonZoomLevel;
-
 struct canonZoomLevelStateStruct {
-	canonZoomLevel value;
+	unsigned char value;
 	char* label;
 };
 
@@ -418,9 +408,9 @@ struct canonExposureBiasStateStruct {
 #define RELEASE_PARAMS_LEN  0x2f
 
 /* These indexes are byte offsets into the release parameter data */
-#define RESOLUTION_1_INDEX  0x01
-#define RESOLUTION_2_INDEX  0x02
-#define RESOLUTION_3_INDEX  0x03
+#define IMAGE_FORMAT_1_INDEX  0x01
+#define IMAGE_FORMAT_2_INDEX  0x02
+#define IMAGE_FORMAT_3_INDEX  0x03
 #define SELF_TIMER_1_INDEX  0x04 /* Currently not used */
 #define SELF_TIMER_2_INDEX  0x05 /* Currently not used */
 #define FLASH_INDEX         0x06
@@ -430,6 +420,7 @@ struct canonExposureBiasStateStruct {
 #define APERTURE_INDEX      0x1c
 #define SHUTTERSPEED_INDEX  0x1e
 #define EXPOSUREBIAS_INDEX  0x20
+#define SHOOTING_MODE_INDEX 0x08
 
 /**
  * canonCaptureSizeClass:
@@ -697,14 +688,16 @@ int canon_int_list_directory (Camera *camera, const char *folder, CameraList *li
 int canon_int_get_file(Camera *camera, const char *name, unsigned char **data, unsigned int *length, GPContext *context);
 int canon_int_get_thumbnail(Camera *camera, const char *name, unsigned char **retdata, unsigned int *length, GPContext *context);
 int canon_int_put_file(Camera *camera, CameraFile *file, char *destname, char *destpath, GPContext *context);
+int canon_int_wait_for_event (Camera *camera, int timeout, CameraEventType *eventtype, void **eventdata, GPContext *context);
 int canon_int_set_file_attributes(Camera *camera, const char *file, const char *dir, canonDirentAttributeBits attrs, GPContext *context);
 int canon_int_delete_file(Camera *camera, const char *name, const char *dir, GPContext *context);
 int canon_int_set_shutter_speed(Camera *camera, canonShutterSpeedState shutter_speed, GPContext *context);
 int canon_int_set_iso(Camera *camera, canonIsoState iso, GPContext *context);
+int canon_int_set_shooting_mode (Camera *camera, unsigned char shooting_mode, GPContext *context);
 int canon_int_set_aperture(Camera *camera, canonApertureState aperture, GPContext *context);
 int canon_int_set_exposurebias(Camera *camera, unsigned char expbias, GPContext *context);
 int canon_int_set_focus_mode (Camera *camera, canonFocusModeState focus_mode, GPContext *context);
-int canon_int_set_resolution (Camera *camera, unsigned char res_byte1, unsigned char res_byte2, unsigned char res_byte3, GPContext *context);
+int canon_int_set_image_format (Camera *camera, unsigned char res_byte1, unsigned char res_byte2, unsigned char res_byte3, GPContext *context);
 int canon_serial_off(Camera *camera);
 int canon_int_get_time(Camera *camera, time_t *camera_time, GPContext *context);
 int canon_int_set_time(Camera *camera, time_t date, GPContext *context);
@@ -715,13 +708,15 @@ int canon_int_start_remote_control(Camera *camera, GPContext *context);
 int canon_int_end_remote_control(Camera *camera, GPContext *context);
 int canon_int_set_beep(Camera *camera, canonBeepMode beep_mode, GPContext *context);
 int canon_int_set_flash(Camera *camera, canonFlashMode flash_mode, GPContext *context);
-int canon_int_set_zoom(Camera *camera, canonZoomLevel zoom_level, GPContext *context);
+int canon_int_set_zoom(Camera *camera, unsigned char zoom_level, GPContext *context);
 
 /*
  * introduced for capturing
  */
 int canon_int_get_release_params (Camera *camera, GPContext *context);
 
+void canon_int_find_new_image ( Camera *camera, unsigned char *initial_state, unsigned char *final_state, CameraFilePath *path );
+
 
 /* path conversion - needs drive letter, and therefore cannot be moved
  * to util.c */
