@@ -32,13 +32,13 @@
 #ifdef OS2
 #include <db.h>
 #endif
-#include <netinet/in.h>
 
+#include "gphoto2-endian.h"
 #include "digita.h"
 
 #define GP_MODULE "digita"
 
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 #define MAX_BEACON_RETRIES	5
 #define MAX_BEACON_TIMEOUTS	2
@@ -122,7 +122,7 @@ static int poll_and_wait(gp_port *dev, int length, int bob, int eob)
 
 static int digita_serial_send(CameraPrivateLibrary *dev, void *_buffer, int len)
 {
-	unsigned char *buffer = _buffer;
+	char *buffer = _buffer;
 	unsigned short s;
 	int sent = 0, size;
 
@@ -170,7 +170,7 @@ static int poll_and_reply(gp_port *dev, int *length, int *eob, int nak)
 
 static int digita_serial_read(CameraPrivateLibrary *dev, void *_buffer, int len)
 {
-	unsigned char *buffer = _buffer;
+	char *buffer = _buffer;
 	unsigned short s;
 	int received = 0, size, eob;
 
