@@ -538,6 +538,8 @@ gp_file_open (CameraFile *file, const char *filename)
             "asf",  GP_MIME_ASF,
             "ogg",  GP_MIME_OGG,
             "mpg",  GP_MIME_MPEG,
+            "mts",  GP_MIME_AVCHD,
+            "m2ts",  GP_MIME_AVCHD,
             NULL};
 
 	CHECK_NULL (file && filename);
@@ -924,6 +926,8 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
 		GP_MIME_WAV,  "wav",
 		GP_MIME_BMP,  "bmp",
 		GP_MIME_AVI,  "avi",
+		GP_MIME_CRW,  "crw",
+		GP_MIME_CR2,  "cr2",
 		NULL};
 
 	CHECK_NULL (file);
@@ -934,11 +938,7 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
                 if (!strcmp (file->mime_type, table[x])) {
 
 			/* Search the current suffix and erase it */
-#ifdef HAVE_STRCHR
 			suffix = strrchr (file->name, '.');
-#else
-			suffix = rindex (file->name, '.');
-#endif
 			if (suffix++)
 				*suffix = '\0';
 			strcat (file->name, table[x + 1]);
