@@ -90,6 +90,8 @@ typedef struct {
 
 typedef int (* begin_func_t)  (const func_params_t *params,
 			       void **data);
+typedef int (* middle_func_t)  (const func_params_t *params,
+			       void **data);
 typedef int (* camera_func_t) (const func_params_t *params, 
 			       const int i,
 			       const int total,
@@ -315,7 +317,7 @@ human_end_func (const func_params_t *params, void *data)
 
 /** C equivalent of basename(1) */
 static const char *
-basename (const char *pathname)
+path_basename (const char *pathname)
 {
 	char *result, *tmp;
 	/* remove path part from camlib name */
@@ -337,7 +339,7 @@ human_camera_func (const func_params_t *params,
 		   void *data)
 {
 	const char *camlib_basename;
-	camlib_basename = basename(a->library);
+	camlib_basename = path_basename(a->library);
 	printf("%3d|%-20s|%-20s|%s\n",
 	       i+1, 
 	       camlib_basename,
@@ -366,12 +368,14 @@ idlist_camera_func (const func_params_t *params,
 
 typedef enum {
 		UDEV_PRE_0_98 = 0,
-		UDEV_0_98 = 1
+		UDEV_0_98 = 1,
+		UDEV_136 = 2
 } udev_version_t;
 
 static const StringFlagItem udev_version_t_map[] = {
 	{ "pre-0.98", UDEV_PRE_0_98 },
 	{ "0.98", UDEV_0_98 },
+	{ "136", UDEV_136 },
 	{ NULL, 0 }
 };
 
@@ -384,6 +388,7 @@ typedef struct {
 	char *script;
 	const char *begin_string;
 	const char *usbcam_string;
+	const char *usbdisk_string;
 } udev_persistent_data_t;
 
 
@@ -395,17 +400,37 @@ udev_parse_params (const func_params_t *params, void **data)
 	 */
 	static const char * const begin_strings[] = {
 		/* UDEV_PRE_0_98 */
-		"BUS!=\"usb_device\", GOTO=\"libgphoto2_rules_end\"\n"
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n",
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"BUS!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n",
 		/* UDEV_0_98 */
-		"SUBSYSTEM!=\"usb|usb_device\", GOTO=\"libgphoto2_rules_end\"\n"
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n"
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb|usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n",
+		/* UDEV_136 */
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
+		"ENV{DEVTYPE}!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n"
+		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{program}=\"usb_id --export %%p\"\n"
+		/* ignore mass storage class having devices in mark-up */
+		"ENV{ID_USB_INTERFACES}==\"*:08*:*\", GOTO=\"libgphoto2_usb_end\"\n"
+		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
+		 * more rules . It will be completed in udev_begin_func() */
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", "
 	};
 	static const char * const usbcam_strings[] = {
 		/* UDEV_PRE_0_98 */
-		"SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\", ",
+		"SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\"",
 		/* UDEV_0_98 */
-		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ",
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\"",
+		/* UDEV_136 */
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\""
+	};
+	static const char * const usbdisk_strings[] = {
+		/* UDEV_PRE_0_98 */
+		"KERNEL==\"%s\", SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\"",
+		/* UDEV_0_98 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\"",
+		/* UDEV_136 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\""
 	};
 	udev_persistent_data_t *pdata;
 	pdata = calloc(1, sizeof(udev_persistent_data_t));
@@ -426,7 +451,8 @@ udev_parse_params (const func_params_t *params, void **data)
 			} else if (strcmp("group", key)==0) {
 				pdata->group = val;
 			} else if (strcmp("version", key)==0) {
-				if (gpi_string_to_enum(val, &(pdata->version),
+				unsigned int *ver = &pdata->version;
+				if (gpi_string_to_enum(val, ver,
 						       udev_version_t_map)) {
 					FATAL("Unrecognized udev version: \"%s\"", val);
 				}
@@ -441,10 +467,11 @@ udev_parse_params (const func_params_t *params, void **data)
 		}
 	}
 	if ((0==0)
-	    && pdata->mode == NULL 
-	    && pdata->group == NULL 
-	    && pdata->owner == NULL 
-	    && pdata->script == NULL) {
+	    && (pdata->mode == NULL)
+	    && (pdata->group == NULL)
+	    && (pdata->owner == NULL)
+	    && (pdata->script == NULL)
+	    && (pdata->version <= UDEV_0_98)) {
 		FATAL("Either <script> or <mode,group,owner> parameters must be given.");
 	}
 	if ((pdata->script != NULL) && (pdata->mode != NULL 
@@ -455,6 +482,7 @@ udev_parse_params (const func_params_t *params, void **data)
 
 	pdata->begin_string = begin_strings[pdata->version];
 	pdata->usbcam_string = usbcam_strings[pdata->version];
+	pdata->usbdisk_string = usbdisk_strings[pdata->version];
 }
 
 
@@ -479,11 +507,35 @@ udev_begin_func (const func_params_t *params, void **data)
 		print_version_comment(stdout, "# ", "\n", NULL, "#\n");
 		printf ("# this file is autogenerated, local changes will be LOST on upgrades\n");
 		printf (pdata->begin_string);
+
+		if (pdata->version == UDEV_136) {
+			if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
+				if (pdata->mode != NULL) {
+					printf("MODE=\"%s\", ", pdata->mode);
+				}
+				if (pdata->owner != NULL) {
+					printf("OWNER=\"%s\", ", pdata->owner);
+				}
+				if (pdata->group != NULL) {
+					printf("GROUP=\"%s\", ", pdata->group);
+				}
+			}
+
+			printf ("GOTO=\"libgphoto2_usb_end\"\n\n");
+		}
 	}
 	return 0;
 }
 
 
+static int
+udev_middle_func (const func_params_t *params, void **data)
+{
+	printf ("\nLABEL=\"libgphoto2_usb_end\"\n\n");
+	return 0;
+}
+
+
 static int
 udev_end_func (const func_params_t *params, void *data)
 {
@@ -547,7 +599,11 @@ udev_camera_func (const func_params_t *params,
 
 	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
 		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) {
-			printf("PROGRAM=\"check-ptp-camera %02d/%02d/%02d\", ", class, subclass, proto);
+			if (pdata->version == UDEV_136) {
+				printf("ENV{ID_USB_INTERFACES}==\"*:%02d%02d%02d:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\"", class, subclass, proto);
+			} else {
+				printf("PROGRAM=\"check-ptp-camera %02d/%02d/%02d\"", class, subclass, proto);
+			}
 			has_valid_rule = 1;
 		} else {
 			if (class == 666) {
@@ -568,6 +624,12 @@ udev_camera_func (const func_params_t *params,
 		}
 	}
 	if (has_valid_rule != 0) {
+		if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
+			printf(", ENV{ID_MEDIA_PLAYER}=\"1\"");
+
+		if (pdata->script != NULL || pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL)
+			printf(", ");
+
 		if (pdata->script != NULL) {
 			printf("RUN+=\"%s\"\n", pdata->script);
 		} else if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
@@ -589,13 +651,69 @@ udev_camera_func (const func_params_t *params,
 			printf("\n");
 		} else {
 			printf("\n");
-			FATAL("udev_camera_func(): illegal branch");
+			if (pdata->version < UDEV_136)
+				FATAL("udev_camera_func(): illegal branch");
 		}
 	}
 	return 0;
 }
 
 
+static int
+udev_camera_func2 (const func_params_t *params, 
+		   const int i,
+		   const int total,
+		   const CameraAbilities *a,
+		   void *data)
+{
+	int has_valid_rule = 0;
+	udev_persistent_data_t *pdata = (udev_persistent_data_t *) data;
+	ASSERT(pdata != NULL);
+
+	if (a->port & GP_PORT_USB_DISK_DIRECT) {
+		printf (pdata->usbdisk_string, "sd[a-z]*",
+			a->usb_vendor, a->usb_product);
+		has_valid_rule = 1;
+	}
+	if (a->port & GP_PORT_USB_SCSI) {
+		printf (pdata->usbdisk_string, "sg[0-9]*",
+			a->usb_vendor, a->usb_product);
+		has_valid_rule = 1;
+	}
+	if (has_valid_rule != 0) {
+		if (pdata->script != NULL || pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL)
+			printf(", ");
+
+		if (pdata->script != NULL) {
+			printf("RUN+=\"%s\"\n", pdata->script);
+		} else if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
+			if (pdata->mode != NULL) {
+				printf("MODE=\"%s\"", pdata->mode);
+				if (pdata->owner != NULL || pdata->group != NULL) {
+					printf(", ");
+				}
+			}
+			if (pdata->owner != NULL) {
+				printf("OWNER=\"%s\"", pdata->owner);
+				if (pdata->group != NULL) {
+					printf(", ");
+				}
+			}
+			if (pdata->group != NULL) {
+				printf("GROUP=\"%s\"", pdata->group);
+			}
+			printf("\n");
+		} else {
+			printf("\n");
+			if (pdata->version < UDEV_136)
+				FATAL("udev_camera_func(): illegal branch");
+		}
+	}
+
+	return 0;
+}
+
+
 static int
 empty_begin_func (const func_params_t *params, void **data)
 {
@@ -706,7 +824,7 @@ ddb_camera_func (const func_params_t *params,
 		 const CameraAbilities *a,
 		 void *data)
 {
-	const char *camlib_basename = basename(a->library);
+	const char *camlib_basename = path_basename(a->library);
 	int head_printed = 0;
 #define DELAYED_HEAD() \
 	do { \
@@ -841,9 +959,14 @@ fdi_camera_func (const func_params_t *params,
 		if (a->usb_vendor) { /* usb product id might be 0! */
 			printf("   <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
 			printf("    <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
+			if (a->usb_vendor == 0x05ac) { /* Apple iPhone, PTP user. */
+				printf("     <match key=\"usb.interface.class\" int=\"6\">\n");
+				printf("      <match key=\"usb.interface.subclass\" int=\"1\">\n");
+				printf("       <match key=\"usb.interface.protocol\" int=\"1\">\n");
+			}
 			if (a->device_type & GP_DEVICE_AUDIO_PLAYER) {
 				printf("     <merge key=\"info.category\" type=\"string\">portable_audio_player</merge>\n");
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
+				printf("     <addset key=\"info.capabilities\" type=\"strlist\">portable_audio_player</addset>\n");
 				printf("     <merge key=\"portable_audio_player.access_method\" type=\"string\">user</merge>\n");
 				printf("     <merge key=\"portable_audio_player.type\" type=\"string\">mtp</merge>\n");
 				
@@ -851,8 +974,8 @@ fdi_camera_func (const func_params_t *params,
 				printf("     <append key=\"portable_audio_player.output_formats\" type=\"strlist\">audio/mpeg</append>\n");
 			} else {
 				printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
-				
+				printf("     <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
+
 				/* HACK alert ... but the HAL / gnome-volume-manager guys want that */
 				if (NULL!=strstr(a->library,"ptp"))
 					printf("     <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
@@ -862,6 +985,11 @@ fdi_camera_func (const func_params_t *params,
 			/* leave them here even for audio players */
 			printf("     <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
 			printf("     <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
+			if (a->usb_vendor == 0x05ac) { /* Apple iPhone */
+				printf("       </match>\n");
+				printf("      </match>\n");
+				printf("     </match>\n");
+			}
 			printf("    </match>\n");
 			printf("   </match>\n");
 			
@@ -870,7 +998,7 @@ fdi_camera_func (const func_params_t *params,
 			printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
 			printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
 			printf("      <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-			printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("      <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
 			if (a->usb_class == 6) {
 				printf("      <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
 			} else {
@@ -1025,6 +1153,8 @@ typedef struct {
 	char *paramdescr;
 	begin_func_t begin_func;
 	camera_func_t camera_func;
+	middle_func_t middle_func;
+	camera_func_t camera_func2;
 	end_func_t end_func;
 } output_format_t;
 
@@ -1071,6 +1201,20 @@ iterate_camera_list (const int add_comments,
 		}
 	}
 
+	if (format->middle_func != NULL) {
+		format->middle_func(&params, &data);
+	}
+
+	if (format->camera_func2 != NULL) {
+		int i;
+		for (i = 0; i < number_of_cameras; i++) {
+			ret = gp_abilities_list_get_abilities (al, i, &a);
+			if (ret < GP_OK)
+				continue;
+			format->camera_func2(&params, i, number_of_cameras, &a, data);
+		}
+	}
+
 	if (format->end_func != NULL) {
 		format->end_func(&params, data);
 	}
@@ -1089,6 +1233,8 @@ static const output_format_t formats[] = {
 	 NULL,
 	 human_begin_func,
 	 human_camera_func,
+	 NULL,
+	 NULL,
 	 human_end_func
 	},
 	{"usb-usermap",
@@ -1099,6 +1245,8 @@ static const output_format_t formats[] = {
 	 "<NAME_OF_HOTPLUG_SCRIPT>",
 	 hotplug_begin_func,
 	 hotplug_camera_func,
+	 NULL,
+	 NULL,
 	 empty_end_func
 	},
 	{"hal-fdi",
@@ -1107,6 +1255,8 @@ static const output_format_t formats[] = {
 	 NULL,
 	 fdi_begin_func,
 	 fdi_camera_func,
+	 NULL,
+	 NULL,
 	 fdi_end_func
 	},
 	{"hal-fdi-device",
@@ -1115,18 +1265,26 @@ static const output_format_t formats[] = {
 	 NULL,
 	 fdi_device_begin_func,
 	 fdi_device_camera_func,
+	 NULL,
+	 NULL,
 	 fdi_device_end_func
 	},
 	{"udev-rules",
-	 "udev rules file either for pre-0.98 or 0.98 and later",
-	 "Put it into /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
+	 "udev rules file",
+	 "For modes \"pre-0.98\" and \"0.98\" (and later), put it into\n"
+	 "        /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
 	 "        or add script to run. This rule files also uses the\n"
 	 "        check-ptp-camera script included in libgphoto2 source. Either put it to\n"
 	 "        /lib/udev/check-ptp-camera or adjust the path in the generated rules file.\n"
-	 "        If you give a script parameter, the mode, owner, group parameters will be ignored.",
+	 "        If you give a script parameter, the mode, owner, group parameters will be ignored.\n"
+	 "        For mode \"136\" put it into /lib/udev/rules.d/40-libgphoto2.rules;\n"
+	 "        you can still use mode/owner/group, but the preferred mode of operation\n"
+	 "        is to use udev-extras for dynamic access permissions.\n",
 	 "[script <PATH_TO_SCRIPT>|version <version>|mode <mode>|owner <owner>|group <group>]*",
 	 udev_begin_func, 
 	 udev_camera_func,
+	 udev_middle_func,
+	 udev_camera_func2,
 	 udev_end_func
 	},
 	{"idlist",
@@ -1135,6 +1293,8 @@ static const output_format_t formats[] = {
 	 NULL,
 	 empty_begin_func, 
 	 idlist_camera_func,
+	 NULL,
+	 NULL,
 	 empty_end_func
 	},
 #ifdef ENABLED_GP2DDB
@@ -1144,6 +1304,8 @@ static const output_format_t formats[] = {
 	 NULL,
 	 ddb_begin_func,
 	 ddb_camera_func,
+	 NULL,
+	 NULL,
 	 ddb_end_func
 	},
 #endif
